@@ -1,14 +1,7 @@
 # coding: utf-8
 
-# Read version from VERSION file
-try:
-    from importlib import resources
-except:
-    from pathlib import Path
-    with open(Path(Path(__file__).resolve().parent, 'VERSION')) as version_file:
-        __version__ = version_file.read().strip()
-else:
-    __version__ = resources.read_text('atomman', 'VERSION').strip()
+# Standard Python imports
+from importlib import resources
 
 # potentials imports
 from potentials import build_lammps_potential, settings
@@ -19,7 +12,7 @@ from . import tools
 from . import mep
 from . import region
 from . import lammps
-from .dump import *
+from .dump import dump, set_dump_styles
 from .dump import __all__ as dump_all
 from .core import *
 from .core import __all__ as core_all
@@ -31,9 +24,15 @@ from .load import __all__ as load_all
 from . import plot
 from . import defect
 
+# Set dump styles
+set_dump_styles()
 
+# Read version from VERSION file
+__version__ = resources.read_text('atomman', 'VERSION').strip()
+
+# Build all list
 __all__ = ['__version__'] + dump_all + core_all + load_all
-__all__ += ['load_lammps_potential', 'build_lammps_potential']
+__all__ += ['load_lammps_potential', 'build_lammps_potential', 'settings']
 __all__.sort()
 
 # Define default working units
