@@ -1,11 +1,15 @@
 # coding: utf-8
 # Standard Python libraries
+from __future__ import annotations
+import io
 from collections import OrderedDict
 from copy import deepcopy
 import warnings
+from typing import Any, Optional, Union, Tuple
 
 # http://www.numpy.org/
 import numpy as np
+import numpy.typing as npt
 
 # https://pandas.pydata.org/
 import pandas as pd
@@ -28,16 +32,19 @@ class System(object):
     
     class _AtomsIndexer(object):
         """Internal class for setitem / getitem acting on atoms"""
-        def __init__(self, host):
+        def __init__(self, host: System):
             self.__host = host
             
-        def __getitem__(self, index):
+        def __getitem__(self,
+                        index: Union[int, list, slice]) -> System:
             """Index getting of Atoms that operates on System."""
             host = self.__host
             return System(atoms=host.atoms[index], box=host.box, pbc=host.pbc,
                           symbols=host.symbols)
             
-        def __setitem__(self, index, value):
+        def __setitem__(self,
+                        index: Union[int, list, slice],
+                        value: Any):
             """Index setting of Atoms that operates on System."""
             host = self.__host
             if isinstance(value, Atoms):
@@ -52,8 +59,15 @@ class System(object):
             else:
                 raise ValueError('Can only set using Atoms or System objects')
     
-    def __init__(self, atoms=None, box=None, pbc=None, scale=False,
-                 symbols=None, masses=None, model=None, safecopy=False):
+    def __init__(self,
+                 atoms: Optional[Atoms] = None,
+                 box: Optional[Box] = None,
+                 pbc: Optional[Tuple[bool, bool, bool]] = None,
+                 scale: bool = False,
+                 symbols: Union[str, list, None] = None,
+                 masses: Union[float, list, None] = None,
+                 model: Union[str, io.IOBase, DM, None] = None,
+                 safecopy: bool = False):
         """
         Initialize a System by joining an am.Atoms and am.Box instance.
         
@@ -170,12 +184,12 @@ class System(object):
         if model is not None:
             for prop in model['atoms'].aslist('property'):
                 if prop['data'].get('unit', None) == 'scaled':
-                    self.atoms.view[prop['name']] = self.unscale(self.atoms.view[prop['name']]) 
+                    self.atoms.view[prop['name']] = self.box.position_relative_to_cartesian(self.atoms.view[prop['name']]) 
         
         # Set atoms indexer
         self.__atoms_ix = System._AtomsIndexer(self)
     
-    def __str__(self):
+    def __str__(self) -> str:
         """str : The string representation of a system."""
         return '\n'.join([str(self.box),
                           'natoms = ' + str(self.natoms),
@@ -184,26 +198,26 @@ class System(object):
                           'pbc = ' + str(self.pbc),
                           str(self.atoms)])
     
-    def __len__(self):
+    def __len__(self) -> int:
         return self.natoms
     
     @property
-    def atoms(self):
+    def atoms(self) -> Atoms:
         """atomman.Atoms : underlying Atoms object."""
         return self.__atoms
     
     @property
-    def natoms(self):
+    def natoms(self) -> int:
         """int : The number of atoms in the Atoms class."""
         return self.__atoms.natoms
     
     @property
-    def atypes(self):
+    def atypes(self) -> tuple:
         """tuple : List of int atom types."""
         return tuple(range(1, self.natypes+1))
     
     @property
-    def natypes(self):
+    def natypes(self) -> int:
         """int : The number of atom types/symbols."""
         
         # Check if more symbols than atypes
@@ -217,28 +231,28 @@ class System(object):
             return self.__atoms.natypes
     
     @property
-    def box(self):
+    def box(self) -> Box:
         """atommman.Box : underlying Box object."""
         return self.__box
     
     @property
-    def pbc(self):
-        """list of bool : The periodic boundary condition settings."""
+    def pbc(self) -> np.ndarray:
+        """numpy.ndarray of bool : The periodic boundary condition settings."""
         return self.__pbc
         
     @pbc.setter
-    def pbc(self, value):
+    def pbc(self, value: npt.ArrayLike):
         pbc = np.asarray(value, dtype=bool)
         assert pbc.shape == (3,), 'invalid pbc entry' 
         self.__pbc = pbc
     
     @property
-    def atoms_ix(self):
+    def atoms_ix(self) -> _AtomsIndexer:
         """Indexer for index slicing of Systems by per-atom properties."""
         return self.__atoms_ix
 
     @property
-    def symbols(self):
+    def symbols(self) -> tuple:
         """tuple : The element model symbols associated with each atype."""
         
         # Fill in missing values
@@ -247,7 +261,7 @@ class System(object):
         return self.__symbols
     
     @symbols.setter
-    def symbols(self, value):
+    def symbols(self, value: Union[str, list]):
         
         # Make value list if needed
         value = aslist(value)
@@ -262,7 +276,7 @@ class System(object):
         self.__symbols = tuple(value)
 
     @property
-    def masses(self):
+    def masses(self) -> tuple:
         """tuple : The masses associated with each atype if given."""
         
         # Fill in missing values
@@ -271,7 +285,7 @@ class System(object):
         return self.__masses
     
     @masses.setter
-    def masses(self, value):
+    def masses(self, value: Union[float, list]):
         
         # Make value list if needed
         value = aslist(value)
@@ -293,7 +307,7 @@ class System(object):
         self.__masses = tuple(value)
     
     @property
-    def composition(self):
+    def composition(self) -> Optional[str]:
         """
         The system's reduced and sorted symbols composition.
         
@@ -301,7 +315,7 @@ class System(object):
         -------
         str or None
             The system's reduced and sorted symbols composition.
-            Will return None if symbols are missing
+            Will return None if any symbols are missing
         """
         
         # Build total count of each symbol
@@ -333,7 +347,13 @@ class System(object):
         
         return composition
     
-    def atoms_prop(self, key=None, index=None, value=None, a_id=None, scale=False):
+    def atoms_prop(self,
+                   key: Optional[str] = None,
+                   index: Union[int, list, slice, None] = None,
+                   value: Optional[Any] = None,
+                   a_id: Optional[int] = None,
+                   scale: bool = False
+                   ) -> Union[list, Atoms, np.ndarray, None]:
         """
         Extends Atoms.prop() by adding a scale argument.
         
@@ -352,6 +372,18 @@ class System(object):
             - Values being retrieved are scaled from absolute Cartesian to box relative vectors.
             - Values being set are unscaled from box relative to absolute Cartesian vectors.
             Default value is False.
+        
+        Returns
+        -------
+        list
+            If no parameters given, returns a list of all assigned property
+            keys.
+        atomman.Atoms
+            If index or a_id is given without value or key, returns a new
+            Atoms instance for the specified atom indices.
+        numpy.ndarray
+            If key (and index/a_id) is given without value, returns a copy of
+            the data associated with that property key.
         """
         
         # Check that scale is bool
@@ -381,7 +413,7 @@ class System(object):
                         newatoms = deepcopy(self.atoms)
                     else:
                         newatoms = deepcopy(self.atoms[index])
-                    newatoms.pos = self.scale(newatoms.pos)
+                    newatoms.pos = self.box.position_cartesian_to_relative(newatoms.pos)
                     return newatoms
                 
                 # If key is given, return scaled property values
@@ -390,7 +422,7 @@ class System(object):
                         value = self.atoms.view[key]
                     else:
                         value = self.atoms.view[key][index]
-                    return self.scale(value)
+                    return self.box.position_cartesian_to_relative(value)
             
             # Set values if value is given
             else:
@@ -399,7 +431,7 @@ class System(object):
                     if not isinstance(value, Atoms):
                         raise TypeError('If key is None, value must be instance of atomman.Atoms')
                     
-                    value.pos = self.unscale(value.pos)
+                    value.pos = self.box.position_relative_to_cartesian(value.pos)
                     if index is None:
                         self.atoms[:] = value
                     else:
@@ -407,13 +439,14 @@ class System(object):
                 
                 # If key is given, unscale value and set to property
                 else:
-                    value = self.unscale(value)
+                    value = self.box.position_relative_to_cartesian(value)
                     if index is None:
                         self.atoms.view[key] = value
                     else:
                         self.atoms.view[key][index] = value
     
-    def atoms_df(self, scale=False):
+    def atoms_df(self,
+                 scale: bool = False) -> pd.DataFrame:
         """
         Extends Atoms.df() by adding a scale argument.
         
@@ -444,7 +477,7 @@ class System(object):
             
             value = self.atoms.view[key]
             if key in scale:
-                value = self.scale(value)
+                value = self.box.position_cartesian_to_relative(value)
             
             # Flatten multidimensional arrays
             for index, istr in indexstr(self.atoms.view[key].shape[1:]):
@@ -459,7 +492,11 @@ class System(object):
         # Return DataFrame
         return pd.DataFrame(values)
     
-    def atoms_extend(self, value, scale=False, symbols=None, safecopy=False):
+    def atoms_extend(self,
+                     value: Union[Atoms, int],
+                     scale: bool = False,
+                     symbols: Optional[list] = None,
+                     safecopy: bool = False) -> System:
         """
         Extends Atoms.extend() to the System level by adding scale and symbols
         arguments.
@@ -494,7 +531,7 @@ class System(object):
         atomman.System
             A new System object with Atoms extended to contain all atoms and
             properties of the current object plus the additional atoms.  The
-            cuurent System object's box and pbc (and symbols if not specified)
+            current System object's box and pbc (and symbols if not specified)
             will be copied over.
         """
         # Copy value if safecopy
@@ -520,7 +557,7 @@ class System(object):
         
         # Unscale pos from Atoms value if needed
         if scale:
-            atoms.pos[value.natoms:] = self.unscale(value.pos)
+            atoms.pos[value.natoms:] = self.box.position_relative_to_cartesian(value.pos)
 
         # Generate and return new System
         return System(atoms=atoms, box=box, pbc=self.pbc, symbols=symbols)
@@ -553,27 +590,38 @@ class System(object):
         else:
             self.box.set(**kwargs)
     
-    def scale(self, value):
+    def scale(self, value: npt.ArrayLike) -> np.ndarray:
         """
         Scales 3D vectors from absolute Cartesian coordinates to relative box
         coordinates.
         
         Parameters
         ----------
-        value : numpy.ndarray
-            Values to scale.
+        value : array-like object
+            Absolute Cartesian coordinates to scale.
+
+        Returns
+        -------
+        numpy.ndarray
+            The relative box coordinates associated with the given values.
         """
+        warnmsg = "System.scale() will likely be depreciated in the next big version update "
+        warnmsg += "as the method name is not informative.  It is being replaced by the "
+        warnmsg += "equivalent Box.position_cartesian_to_relative() method."
+        warnings.warn(warnmsg, PendingDeprecationWarning)
         
         # Retrieve parameters
-        value = np.asarray(value, dtype=float)
-        vects = self.box.vects
-        inverse = np.linalg.inv(vects)
-        origin = self.box.origin
+        #value = np.asarray(value, dtype=float)
+        #vects = self.box.vects
+        #inverse = np.linalg.inv(vects)
+        #origin = self.box.origin
         
         # Convert
-        return (value - origin).dot(inverse)
-          
-    def unscale(self, value):
+        #return (value - origin).dot(inverse)
+
+        return self.box.position_cartesian_to_relative(value)
+
+    def unscale(self, value: npt.ArrayLike) -> np.ndarray:
         """
         Unscales 3D vectors from relative box coordinates to absolute
         Cartesian coordinates.
@@ -581,18 +629,30 @@ class System(object):
         Parameters
         ----------
         value : numpy.ndarray
-            Values to unscale.
+            Relative box coordinates to unscale.
+
+        Returns
+        -------
+        numpy.ndarray
+            Absolute Cartesian coordinates associated with the given values.
         """
+        warnmsg = "System.unscale() will likely be depreciated in the next big version update "
+        warnmsg += "as the method name is not informative.  It is being replaced by the "
+        warnmsg += "equivalent Box.position_relative_to_cartesian() method."
+        warnings.warn(warnmsg, PendingDeprecationWarning)
         
         # Retrieve parameters
-        value = np.asarray(value, dtype=float)
-        vects = self.box.vects
-        origin = self.box.origin
+        #value = np.asarray(value, dtype=float)
+        #vects = self.box.vects
+        #origin = self.box.origin
         
         # Convert
-        return value.dot(vects) + origin
+        #return value.dot(vects) + origin
+
+        return self.box.position_relative_to_cartesian(value)
         
-    def wrap(self, return_imageflags=False):
+    def wrap(self,
+             return_imageflags: bool = False) -> Optional[np.ndarray]:
         """
         Wrap atoms around periodic boundaries and extend non-periodic
         boundaries such that all atoms are within the box.
@@ -606,7 +666,7 @@ class System(object):
         Returns
         -------
         numpy.NDarray of int
-            The imageflags array
+            The imageflags array - only returned if return_imageflags = True.
         """
         
         # mins and maxs are box dimensions relative to box vectors, i.e 0 to 1
@@ -651,18 +711,21 @@ class System(object):
         if return_imageflags:
             return imageflags
     
-    def dvect(self, pos_0, pos_1):
+    def dvect(self,
+              pos_0: Union[int, list, slice, npt.ArrayLike],
+              pos_1: Union[int, list, slice, npt.ArrayLike]
+              ) -> np.ndarray:
         """
         Computes the shortest vector between pos_0 and pos_1 using box 
         dimensions and accounting for periodic boundaries.
         
         Parameters
         ----------
-        pos_0 : numpy.ndarray or index
+        pos_0 : index or array-like object
             Absolute Cartesian vector position(s) to use as reference point(s).
             If the value can be used as an index, then self.atoms.pos[pos_0]
             is taken.
-        pos_1 : numpy.ndarray or index
+        pos_1 : index or array-like object
             Absolute Cartesian vector position(s) to find relative to pos_0.
             If the value can be used as an index, then self.atoms.pos[pos_1]
             is taken.
@@ -689,18 +752,21 @@ class System(object):
         else:
             return vects
 
-    def dmag(self, pos_0, pos_1):
+    def dmag(self,
+             pos_0: Union[int, list, slice, npt.ArrayLike],
+             pos_1: Union[int, list, slice, npt.ArrayLike]
+             ) -> np.ndarray:
         """
         Computes the shortest distance between pos_0 and pos_1 using box 
         dimensions and accounting for periodic boundaries.
         
         Parameters
         ----------
-        pos_0 : numpy.ndarray or index
+        pos_0 : index or array-like object
             Absolute Cartesian vector position(s) to use as reference point(s).
             If the value can be used as an index, then self.atoms.pos[pos_0]
             is taken.
-        pos_1 : numpy.ndarray or index
+        pos_1 : index or array-like object
             Absolute Cartesian vector position(s) to find relative to pos_0.
             If the value can be used as an index, then self.atoms.pos[pos_1]
             is taken.
@@ -727,7 +793,7 @@ class System(object):
         else:
             return vects
     
-    def neighborlist(self, **kwargs):
+    def neighborlist(self, **kwargs) -> NeighborList:
         """
         Builds a neighbor list for the system.  The resulting NeighborList
         object is saved to the object as attribute 'neighbors'.
@@ -759,7 +825,8 @@ class System(object):
             kwargs['system'] = self
         return NeighborList(**kwargs)
     
-    def r0(self, neighbors=None):
+    def r0(self,
+           neighbors: Optional[NeighborList] = None) -> float:
         """
         Identifies the shortest interatomic spacing between atoms in the
         system by comparing the shortest periodic box vector with the
@@ -822,7 +889,10 @@ class System(object):
         else:
             raise ValueError('No atoms to compare or periodic boundaries found!')
     
-    def supersize(self, a_size, b_size, c_size):
+    def supersize(self,
+                  a_size: Union[int, Tuple[int, int]],
+                  b_size: Union[int, Tuple[int, int]],
+                  c_size: Union[int, Tuple[int, int]]) -> System:
         """
         Creates a larger system from a given system by replicating it along the
         system's box vectors.
@@ -955,7 +1025,10 @@ class System(object):
         
         return System(box=box, atoms=atoms, scale=True, symbols=self.symbols)
     
-    def rotate(self, uvws, tol=None, return_transform=False):
+    def rotate(self,
+               uvws: npt.ArrayLike,
+               tol: Union[float, list, None] = None,
+               return_transform: bool = False) -> System:
         """
         Transforms a System representing a periodic crystal cell from a standard
         orientation to a specified orientation. Note: if hexagonal indices are
@@ -964,7 +1037,7 @@ class System(object):
         
         Parameters
         ----------
-        uvws : numpy.ndarray
+        uvws : array-like object
             A (3, 3) array of the Miller crystal vectors or a (3, 4) array of
             Miller-Bravais hexagonal crystal vectors to use in transforming the
             system.  Values must be integers.
@@ -1011,68 +1084,76 @@ class System(object):
         else:
             raise ValueError('Rotation uvws must be integer values')
         
-        # Get natoms and volume of system
-        natoms = self.natoms
-        volume = self.box.volume
+        # No rotation shortcut
+        if np.all(uvws == np.eye(3, dtype='int64')):
+            newsystem = deepcopy(self)
         
-        # Convert uvws to Cartesian units and compute new volume and natoms
-        newvects = miller.vector_crystal_to_cartesian(uvws, box=self.box)
-        newvolume = np.abs(newvects[0].dot(np.cross(newvects[1], newvects[2])))
-        newnatoms = int(round(newvolume / volume) * natoms)
-        
-        # Check new values
-        if newnatoms == 0:
-            raise ValueError('New box has no atoms/volume: vectors are parallel or planar')
-        
-        # Identify box corners of new system wrt uvws
-        corners = np.empty((8,3), dtype='int64')
-        corners[0] = np.zeros(3)
-        corners[1] = uvws[0]
-        corners[2] = uvws[1]
-        corners[3] = uvws[2]
-        corners[4] = uvws[0] + uvws[1]
-        corners[5] = uvws[0] + uvws[2]
-        corners[6] = uvws[1] + uvws[2]
-        corners[7] = uvws[0] + uvws[1] + uvws[2]
-        
-        # Create a supercell of system that contains all box corners
-        a_mults = (corners[:,0].min()-1, corners[:,0].max()+1)
-        b_mults = (corners[:,1].min()-1, corners[:,1].max()+1)
-        c_mults = (corners[:,2].min()-1, corners[:,2].max()+1)
-        system2 = self.supersize(a_mults, b_mults, c_mults)
-        
-        # Change system.box.vects to newvects
-        system2.box_set(vects=newvects, scale=False)
-        
-        search_success = False
-        for atol in tol:
+        else:
+            # Get natoms and volume of system
+            natoms = self.natoms
+            volume = self.box.volume
             
-            spos = system2.atoms_prop('pos', scale=True)
+            # Convert uvws to Cartesian units and compute new volume and natoms
+            newvects = miller.vector_crystal_to_cartesian(uvws, box=self.box)
+            newvolume = np.abs(newvects[0].dot(np.cross(newvects[1], newvects[2])))
+            newnatoms = int(round(newvolume / volume) * natoms)
             
-            # Round atom positions near box boundaries to the boundaries
-            spos[np.isclose(spos, 0.0, atol=atol)] = 0.0
-            spos[np.isclose(spos, 1.0, atol=atol)] = 1.0
-        
-            # Identify all atoms whose positions are 0 <= x < 1
-            aindex = np.where(((spos[:, 0] >= 0.0) & (spos[:, 0] < 1.0)
-                             & (spos[:, 1] >= 0.0) & (spos[:, 1] < 1.0)
-                             & (spos[:, 2] >= 0.0) & (spos[:, 2] < 1.0)))
+            # Check new values
+            if newnatoms == 0:
+                raise ValueError('New box has no atoms/volume: vectors are parallel or planar')
             
-            # Check if number of atoms identified matches the expected number
-            if len(aindex[0]) == newnatoms:
-                search_success = True
-                break
-        
-        if not search_success:
-            raise ValueError(f'Filtering failed: {newnatoms} atoms expected, {len(aindex[0])} found')
-        
-        # Make newsystem by cutting out all atoms in system2 outside boundaries
-        newsystem = System(atoms=system2.atoms[aindex], box=system2.box, symbols=self.symbols)
+            # Identify box corners of new system wrt uvws
+            corners = np.empty((8,3), dtype='int64')
+            corners[0] = np.zeros(3)
+            corners[1] = uvws[0]
+            corners[2] = uvws[1]
+            corners[3] = uvws[2]
+            corners[4] = uvws[0] + uvws[1]
+            corners[5] = uvws[0] + uvws[2]
+            corners[6] = uvws[1] + uvws[2]
+            corners[7] = uvws[0] + uvws[1] + uvws[2]
+            
+            # Create a supercell of system that contains all box corners
+            a_mults = (corners[:,0].min()-1, corners[:,0].max()+1)
+            b_mults = (corners[:,1].min()-1, corners[:,1].max()+1)
+            c_mults = (corners[:,2].min()-1, corners[:,2].max()+1)
+            system2 = self.supersize(a_mults, b_mults, c_mults)
+            
+            # Change system.box.vects to newvects
+            system2.box_set(vects=newvects, scale=False)
+            
+            search_success = False
+            for atol in tol:
+                
+                spos = system2.atoms_prop('pos', scale=True)
+                
+                # Round atom positions near box boundaries to the boundaries
+                spos[np.isclose(spos, 0.0, atol=atol)] = 0.0
+                spos[np.isclose(spos, 1.0, atol=atol)] = 1.0
+            
+                # Identify all atoms whose positions are 0 <= x < 1
+                aindex = np.where(((spos[:, 0] >= 0.0) & (spos[:, 0] < 1.0)
+                                 & (spos[:, 1] >= 0.0) & (spos[:, 1] < 1.0)
+                                 & (spos[:, 2] >= 0.0) & (spos[:, 2] < 1.0)))
+                
+                # Check if number of atoms identified matches the expected number
+                if len(aindex[0]) == newnatoms:
+                    search_success = True
+                    break
+            
+            if not search_success:
+                raise ValueError(f'Filtering failed: {newnatoms} atoms expected, {len(aindex[0])} found')
+            
+            # Make newsystem by cutting out all atoms in system2 outside boundaries
+            newsystem = System(atoms=system2.atoms[aindex], box=system2.box, symbols=self.symbols)
         
         # Return normalized system
         return newsystem.normalize(return_transform=return_transform)
     
-    def normalize(self, style='lammps', return_transform=False):
+    def normalize(self,
+                  style: str = 'lammps',
+                  return_transform: bool = False
+                  ) -> Union[System, Tuple[System, np.ndarray]]:
         """
         Normalizes a system's box vectors and atom positions to be compatible
         with simulation codes.
@@ -1098,7 +1179,7 @@ class System(object):
         else:
             raise ValueError("Unknown style (only 'lammps' is currently supported)")
     
-    def dump(self, style, **kwargs):
+    def dump(self, style: str, **kwargs) -> Optional[Any]:
         """
         Convert a System to another format.
     
@@ -1116,7 +1197,11 @@ class System(object):
         """
         return dump(style, self, **kwargs)
 
-    def model(self, box_unit=None, prop_name=None, unit=None, prop_unit=None):
+    def model(self,
+              box_unit: Optional[str] = None,
+              prop_name: Optional[list] = None,
+              unit: Optional[list] = None,
+              prop_unit: Optional[dict] = None) -> DM:
         """
         Generates a data model for the System object.
 
@@ -1177,6 +1262,6 @@ class System(object):
         # Scale properties if needed
         for prop in amodel.aslist('property'):
             if prop['data'].get('unit', None) == 'scaled':
-                prop['data'] = uc.model(self.scale(uc.value_unit(prop['data'])), units='scaled') 
+                prop['data'] = uc.model(self.box.position_cartesian_to_relative(uc.value_unit(prop['data'])), units='scaled') 
         
         return model
