@@ -6,6 +6,7 @@ from potentials.tools.atomic_info import *
 from potentials.tools.atomic_info import __all__ as atomic_info_all
 
 # atomman imports
+from .boolean import boolean
 from .axes_check import axes_check
 from .compositionstr import compositionstr
 from .duplicates_allclose import duplicates_allclose
@@ -16,7 +17,7 @@ from .crystalsystem import *
 from .crystalsystem import __all__ as crystalsystem_all
 from . import miller
 
-__all__ = ['axes_check', 'compositionstr', 'duplicates_allclose', 'vect_angle',
+__all__ = ['axes_check', 'boolean', 'compositionstr', 'duplicates_allclose', 'vect_angle',
            'uber_open_rmode', 'indexstr', 'filltemplate', 'miller', 'aslist',
            'iaslist', 'screen_input']
 __all__.extend(atomic_info_all)
